package uk.ac.shef.is.imageClef.cluster;

import java.io.File;
import java.util.HashMap;

import uk.ac.shef.is.imageClef.clusterAssessment.ClusterTopicAssessmentsContainer;
import uk.ac.shef.is.imageClef.io.InputOutput;
import uk.ac.shef.is.imageClef.qrel.QrelsContainer;
import uk.ac.shef.is.imageClef.trecResult.TrecResultsContainer;
import uk.ac.shef.is.imageClef.trecResult.TrecTopicResultsContainer;

public class ClusterPrecision {
	
	/** Method return to minimal Rank of a given cluster recall r (0<r<1)
	 * @param requiredNumberOfUniqueClusters Minimal of number of required unique clusters
	 * @param topicNr topic
	 * @param trc Container of the TREC-Style result file
	 * @param qrelsContainer Container of the QREL file
	 * @param clusterAssessmentContainer Container of the cluster assessments
	 * @return minimal rank, which coveres the minimal required number of unique clusters
	 */
	public static int getClusterRank(int requiredNumberOfUniqueClusters, String topicNr, TrecResultsContainer trc, QrelsContainer qrelsContainer, HashMap<String, ClusterTopicAssessmentsContainer> clusterAssessmentContainer){
		String runName = Cluster.getRunName(trc.getFilePath()); //returns proper runName
		HashMap<String, Integer>  numberOfUniqueClusters = new HashMap<String, Integer>(); //to store the covered clusters
		if(trc.hasTopicResults(topicNr)){
			ClusterTopicAssessmentsContainer ctac = clusterAssessmentContainer.get(topicNr);
			
			TrecTopicResultsContainer topicResults = trc.getTopicResults(topicNr);
			int numberOfTrecResultsInSelectedTopic = topicResults.getNumberOfResults(); //Number of results for the current topic
			for(int rank = 0; rank<numberOfTrecResultsInSelectedTopic; rank++){
				String docNr = topicResults.getDocByRank(""+rank); //returns document at the current rank
				if(qrelsContainer.isRelevant(topicNr, docNr)){ //check whether current document is relevant accoring to qrel-file
					String clusterId = ctac.getClusterIdByDocNrAndTopicNr(docNr); // get clusterId of current document
					numberOfUniqueClusters.put(clusterId, 1);
					if(numberOfUniqueClusters.size() >= requiredNumberOfUniqueClusters){
						System.out.println("R:"+rank+" || DocNr:"+docNr + " || ClusterId:"+ clusterId + " || Topic:" + topicNr + " || File:"+trc.getFilePath());
						return rank+1; //because trec-ranking starts with 0
					}
					else{
//						System.out.println("R:"+rank+" || DocNr:"+docNr + " || ClusterId:"+ clusterId + " || Topic:" + topicNr + " || RunName:"+runName);
//						System.out.println("unique covered clusters / required number of unique clusters :" +numberOfUniqueClusters.size() + "/" + requiredNumberOfUniqueClusters );
					}
				}
			}
		}
		else{
			System.err.println("Error CP: Requested topic >" + topicNr + "< could not be found in run : " +runName);
			if(requiredNumberOfUniqueClusters == 20) InputOutput.writeFileContent(new File("/home/thomas/ImageClef/Submissions/Errors/ClusterPrecision_Error.txt"), "Error CP : Requested topic >" + topicNr + "< could not be found in run : " +runName  + "\n", true);
		}
		return -1;
	}
}