package uk.ac.shef.is.imageClef.consitency;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;

import uk.ac.shef.is.imageClef.trecResult.TrecResultEntryContainer;
import uk.ac.shef.is.imageClef.trecResult.TrecResultParser;

public class Consitency {
	
	/** Make some check on the submission file
	 * @param trecFile File to check
	 * @param checkDecreasingScores Set true if file should be check for  strictly decreasing scores 
	 * @return true if file is fine
	 */
	public static boolean checkTrecFile(File trecFile, boolean checkDecreasingScores){	
		//HashMap to check if TopicNr in combination with DocNr are unique
		HashMap<String, String> unique = new HashMap<String, String>();
		try {
			BufferedReader input = new BufferedReader(new InputStreamReader(new FileInputStream(trecFile)));
			try {
				String line = null; //not declared within while loop
				int lastTopicNr = 1; //to store the last topic number
				double lastTopicScore = Double.MAX_VALUE; //to reset after each new topic
				int counter = 0;			
				
				while ((line = input.readLine()) != null) {
					TrecResultEntryContainer trec = TrecResultParser.parseLine(line, trecFile);
					
					int currentTopicNr = Integer.parseInt(trec.getTopicNr());
					double currentScore = Double.parseDouble(trec.getScorce());

					//set the first time the current topic number as the last one
					//this is important when the topic does not start with 1
					if(counter ==0){
						lastTopicNr=currentTopicNr;
					}
					counter++;
					
					//check ascending topic number
					if(currentTopicNr < lastTopicNr){ //topic Nr from each line must be equal or bigger
						System.err.println("WARNING : Topic number not in correct order (should be in ascending order): " + line + " in file " + trecFile.getAbsoluteFile());
						return false;
					}					
					

					if(isNewTopicNr(currentTopicNr,lastTopicNr)){
						//set the highest score after each new topic that
						//the score of rank 1 is certainly
						lastTopicScore = Double.MAX_VALUE;
					}
					lastTopicNr = currentTopicNr; //must be after checking for new topic
					
					//check decreasing scores
					if(checkDecreasingScores){ //check only if specified
						if(currentScore >= lastTopicScore ){
							System.err.println("_____________________________");
							System.err.println("WARNING : Document scores not in correct order (should be in desceding order): " + line + " in file " + trecFile.getAbsoluteFile());
							System.err.println("lastScore="+lastTopicScore + "   CurrentScore=" +currentScore );
							System.err.println("-------------------------------");
							return false;
						}
						lastTopicScore = currentScore;
					}

					//check uniqueness
					String uniqueKey = trec.getTopicNr() + "-" + trec.getDocNr();
					if(unique.containsKey(uniqueKey)){
						System.err.println("DocNr must be unique within one topic: >" + line + "< in file " + trecFile.getAbsoluteFile());
						return false;
					}
					else unique.put(uniqueKey, null);
					//end chech uniqueness
				}
			} finally {
				input.close();
			}
		} catch (IOException ex) {
			ex.printStackTrace();
			return false;
		}		
		return true;
	}
	
	
	
	private static boolean isNewTopicNr(int currentTopicNumber, int lastTopicNumber){
//		System.out.println(lastTopicNumber + "  -  " + currentTopicNumber);
		if(currentTopicNumber != lastTopicNumber) return true;
		else return false;
	}
}
