package uk.ac.shef.is.imageClef.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;

/** Class handles basic input output functions
 * @author Thomas Arni
 *
 */
public class InputOutput {
	
	/** Return content of the provide file
	 * @param file File to read
	 * @return Content of file
	 */
	public static String getContents(File file) {
		StringBuffer contents = new StringBuffer();
		try {
			BufferedReader input = new BufferedReader(new InputStreamReader(new FileInputStream(file),"ISO-8859-15"));
			try {
				String line = null; //not declared within while loop
				while ((line = input.readLine()) != null) {
					contents.append(line);
					contents.append(System.getProperty("line.separator"));
				}
			} finally {
				input.close();
			}
		} catch (IOException ex) {
			System.out.println("Error during reading file : " + file.getAbsoluteFile());
			return "no content";
		}
		return contents.toString();
	}
	
	/** Write content to a file
	 * @param file File to write
	 * @param content Content to write in file
	 * @param append If content should be append or not
	 */
	public static void writeFileContent(File file, String content, boolean append){
	    try {
	        BufferedWriter out = new BufferedWriter(new FileWriter(file, append));
	        out.write(content);
	        out.close();
	    } catch (IOException e) {
	    	System.err.println("Error during writing File : "+ file.getAbsolutePath());    	
	    }
	}
	
	/** Writes content to a specific file. Creates folder and file if is does not exists.
	 * @param content Content to write in file
	 * @param folder Folder to write the file
	 * @param filename name of the file
	 */
	public static void wirteResultFile(String content,String folder, String filename){
		try{
			File docDir = new File(folder);
			if ( !docDir.exists() ) { //create new folder if it does not exists yet
				new File(folder).mkdir();
			}
			File f= new File(folder + "/" + filename);
			if(f.exists()){
				System.err.println("File already exists :" +f.getAbsolutePath());
			}
			
			FileWriter fstream = new FileWriter(folder +"/"+ filename, false);
			BufferedWriter out = new BufferedWriter(fstream);
			out.write(content + "\n");
			out.close();
		}catch (Exception e){//Catch exception if any
			System.err.println("Exception during writing resultfile: " + e.getMessage());
		}	  
	}
}