package uk.ac.shef.is.imageClef.trecResult;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import uk.ac.shef.is.imageClef.qrel.QrelsContainer;


/** Class is a container for a whole trec result file
 * @author Thomas Arni
 *
 */
public class TrecResultsContainer {
	//Container<TopicNr,<rank, TrecResultEntry> >
	private HashMap<String, TrecTopicResultsContainer> results = new HashMap<String, TrecTopicResultsContainer>();
	private String filePath = null;
	private String runName = null;
	
	
	/** Return true if the requested topic number has entries
	 * @param topicNr Requested topic number
	 * @return true if trec result file has documents for the requested topic
	 */
	public boolean hasTopicResults(String topicNr){
		return results.containsKey(topicNr);
	}
	
	/** Return the existing TrecTopicResultsContainer or create a new one 
	 * @param topicNr Requested Topic
	 * @return Existing or new TrecTopicResultsContainer
	 */
	public TrecTopicResultsContainer getTopicResults(String topicNr){
		if(results.containsKey(topicNr)){
			return results.get(topicNr);
		}
		else{
			TrecTopicResultsContainer ttr = new TrecTopicResultsContainer(topicNr);
			results.put(topicNr, ttr);
			return ttr;
		}
	}	
	
	/** Returns the document number of a specific topic and rank
	 * @param topicNr Request topic
	 * @param rank Requested rank
	 * @return Document number from the requested topic at the requested rank
	 */
	public String getDocNrByTopicNrAndRank(String topicNr, String rank){
		return results.get(topicNr).getDocByRank(rank);
	}

	public String getFilePath() {
		return filePath;
	}

	/** Set the file path of trec result file. This is not crucial, however it makes
	 * debugging much easier if you can see in which file errors occurred.
	 * @param filePath Path to the trec result file.
	 */
	public void setFilePath(String filePath) {
		this.filePath = filePath;
	}
	
	/** Returns the number of topics
	 * @return number of topics
	 */
	public int getNumberOfTopics(){
		return results.size();
	}
	
	public int getNumberOfValidTopics(QrelsContainer qrelsContainer){
		int numberOfValidTopics = 0;
		String[] topics = this.getTopicNumbers();
		for (String topic : topics) {
			if(qrelsContainer.isValidTopicNr(topic)){
				numberOfValidTopics++;
			}			
		}
		return numberOfValidTopics;
	}
	
	/** Return the occurred topic number in of the trec result in a ascending order
	 * @return topic Numbers
	 */
	public String[] getTopicNumbers(){
		String[] topicNrs = new String[results.size()];
		Set<Map.Entry<String,TrecTopicResultsContainer>> s =results.entrySet();
		Iterator<Map.Entry<String,TrecTopicResultsContainer>> it = s.iterator();
		int counter = 0;
		while(it.hasNext()){
			Map.Entry<String, TrecTopicResultsContainer> entry = it.next();
			topicNrs[counter] = entry.getKey();
			counter++;
		}		
		Comparator<String> myOrder = new Comparator<String>(){
			   public int compare(String o1, String o2) {
			      int r1 = Integer.parseInt(o1);
			      int r2 = Integer.parseInt(o2);
			      if (r1 < r2) return -1;
			      else if (r1 > r2) return 1;
			      return 0;
			   }
		};
		java.util.Arrays.sort(topicNrs,myOrder);
		return topicNrs;
	}

	/** Return the Trec Result with decreasing scores and increasing ranks
	 * @return Trec Result with decreasing scores
	 */
	public String getTrecFileWithDecreasinigScore(){
		StringBuffer sb = new StringBuffer();
		String[] topics = this.getTopicNumbers(); //get topics from current trec result
		for (String topic : topics) {
			TrecTopicResultsContainer ttrc = results.get(topic);
			HashMap<String, TrecResultEntryContainer> topicResults = ttrc.getTopicResults();
			int score =1000;
			for(int rank = 0; rank < topicResults.size(); rank++){
				if(topicResults.containsKey(""+rank)){
					TrecResultEntryContainer tte = topicResults.get(""+rank);
					sb.append(tte.getTopicNr() + " 1 " + tte.getDocNr() + " " + rank + " " + score-- + " " + tte.getRunName() + "\n");
				}
			}
		}
		return sb.toString().trim(); //trim to remove last added new line
	}

	public void setRunName(String runName) {
		this.runName = runName;
	}

	public String getRunName() {
		return runName;
	}
}